#include "ti86asm.inc"
#include "crasm86.inc"

;File Manager v1.2 revision 4
;by Cassady Roop

dynamic_data	equ $E800
tempbyte		equ	dynamic_data+0	;just a temp storage byte
var_name		equ	dynamic_data+1	;name of var (type,length,8 byte name, not z-termed)
new_name		equ dynamic_data+11
var_length		equ	dynamic_data+21	;length header of var.  2 bytes
var_firstbyte	equ	dynamic_data+23	;first byte of var data
var_secondbyte	equ	dynamic_data+24	;second byte of var data
listcounter		equ	dynamic_data+25	;a counter for reading the list
selected		equ	dynamic_data+26	;currently highlighted var number
filter_type		equ dynamic_data+27	;variable type filter
num_pages		equ	dynamic_data+28	;total number of screen pages
cur_page		equ	dynamic_data+29	;current page
vatlist_entries	equ	dynamic_data+30	;number of variables in the vatlist
pagelist_entries	equ	dynamic_data+31	;number of items in the pagelist
pagelist		equ	dynamic_data+32	;portion of vatlist corresponding to current page. 210 bytes, 21*10 B/entry
vatlist			equ	pagelist+210	;a list of all prgm/strn vars on the calc.  May take up to about 2Kbytes.

anticrash		equ		7			;bit 7 of <flagbyte> is the anticrash on/off flag


.org _asm_exec_ram

ProgStart:								;program startup
	ld hl, launcher_name-1
	rst 20h
	rst 10h
	jp c, create_launcher				;make a launcher
	call _cursoroff						;turn off cursor
	call _runindicoff
	call _flushallmenus
	ld a, $12							;type byte for prgm
	ld (filter_type), a					;show only prgm vars first time around
	xor a
	ld (cur_page), a					;start on display page 0

top:
	call clrLaunchExecName				;clean slate in the launch code
	xor a
	ld (vatlist_entries), a				;clean slate
	ld (pagelist_entries), a
	ld (cur_page), a					;page 0
	ld hl, searchbase					;begin searching from this name
	ld a, (filter_type)					;get filter byte
	and %00000001						;I'm only after bit 0
	cp 0								;is it odd or even?
	jr nz, top_odd
	ld a, $12							;if even, show prgm variables
	jr top_filterdone
top_odd:
	ld a, $0C							;if odd, show string variables
top_filterdone:
	ld (hl), a							;load it to the type byte of the searchbase
	call search							;searches the VAT for variables of type 'filter_byte'
	ld a, (vatlist_entries)
	cp 0								;if search returns a 'none-found'
	jr nz, vars_present
	ld a, $FF
	ld (vatlist_entries), a				;FF will mean no var found
vars_present:	
	call findpages						;determines and stores the number of pages this list takes to display
	call pagify							;makes a page-specific, small, disp-able var list from the current page and vatlist
	xor a
	ld (selected), a					;upper left (entry 0) is selected
	call display						;displays user interface, and the vatlist, and current selection
	jp keyloop							;handles user input
	

search:								;search VAT for variables
	rst 20h							;move ten byte name at hl to OP1
	xor a							;a must be zero to search by type
	call _findAlphaUp				;returns the next highest program
	ret c							;if c, then there are no more vars to list
	ld hl, _OP1						;_findalphup returned the name in OP1
	ld de, var_name					;we will copy it here
	ld bc, $000A					;we want all ten bytes of OP1
	ldir							;block copy
	ld a, (vatlist_entries)			;current number of entries stored
	ld l, a
	ld h, 0							;hl = $00[vatlist_entries]
	call _mulHL10					;hl = hl*10
	ld de, vatlist					;start of the list
	add hl, de						;add the offset
	ex de, hl						;put the new entry address in de
	ld hl, var_name					;point to the name to store
	ld bc, $000A					;ten bytes
	ldir							;copy to the vatlist
	ld a, (vatlist_entries)			;get old number of entries
	inc a
	ld (vatlist_entries), a			;update it
	ld hl, var_name
	jr search						;reloop, using this name as a search base
	

findpages:							;determines and stores the number of pages the list will take to display
	ld a, (vatlist_entries)			;number of list entries
	ld c, $FF						;counter (will be incremented to $00)
fpgs_loop:
	inc c							;increment counter
	sub 21							;21 entries fit on a screen page
	jr z, fpgs_found				;if there are 21 exactly, we are done
	jp p, fpgs_loop					;if positive, there is another page
fpgs_found:							;we have counted the pages
	ld a, c							;get counter value
	sub 8
	jp p, fpgs_overflow				;too many pages.  I will only display 9 (0-8).
	ld a, c
	ld (num_pages), a
	ret								;done
fpgs_overflow:
	ld a, 8
	ld (num_pages), a				;set to maximum
	ret


pagify:								;create pagelist
	ld a, (vatlist_entries)
	cp $FF							;FF means empty
	jr nz, pag_continue
	ld a, $FF
	ld (pagelist_entries), a		;synch with vatlist counter
pag_continue:
	ld hl, pagelist
	ld b, 210
pag_clearloop:						;clear the pagelist
	ld (hl), 0
	inc hl
	djnz pag_clearloop
	ld a, (cur_page)				;page we want to generate list for
	ld d, a
	rlca							;*2
	ld b, a							;b=*2
	rlca							;*4
	rlca							;*8
	rlca							;*16
	add a, b						;(*2)+(*16)=(*18)
	ld c, a							;c=*18
	ld a, b
	add a, d
	add a, c						;(*4)+(*18)=(*22)
	ld h, 0
	ld l, a							;page*21
	push af
	call _mulHL10					;page*210
	ex de, hl
	ld hl, vatlist
	add hl, de						;point hl to first byte of first entry of current page
	pop bc							;pushed af, popped into bc
	ld a, (vatlist_entries)			;number of total entries
	sub b							;subtract start of page from max to get number of entries on that page
	ld b, a
	sub 21							;21 per page
	jp m, pag_copy					;if neg, then less than 21 left.  Correct number is already in reg. b
pag_set21:							;either exactly 21 left, or more left, so make a full page
	ld b, 21						;entry countdown counter
pag_copy:
	xor a							;entry countup counter to store to pagelist_entries
	ld de, pagelist					;destination
	ld c, 10
pag_copyloop:
	push af
	ld a, b
	cp 0
	jr z, pag_copydone
	pop af
	push bc
	ld b, 0
	ldir							;copy from hl to de, ten bytes
	inc a							;increment 
	pop bc							;recall my counter
	djnz pag_copyloop				;loop until all entries are copied
	push hl
pag_copydone:
	pop hl
	ld (pagelist_entries), a		;record number of entries copied
	ret



	
display:							;display interface and var list
	call _clrLCD
	ld hl, $0000
	ld (_pencol), hl
	ld hl, titletext
	call _vputs						;display title
	ld hl, $004B
	ld (_pencol), hl
	ld hl, credittext
	call _vputs						;'by Cassady Roop'
	ld hl, $FC70					;row 7
	ld b, 16
display_lineloop:
	ld (hl), $FF					;8 dark pixels
	inc hl
	djnz display_lineloop			;do it 16 times for a whole line
	call feedback					;user feedback at bottom of screen
	ld a, (pagelist_entries)
	cp $FF
	ret z							;nothing to display
	call highlight					;make black bar at selection
	xor a
	ld (listcounter), a
	ld hl, $0301
	ld (_pencol), hl				;initial screen coords
disploop1:							;handles column 1
	ld a, (_penRow)					;current row coord
	add a, $06						;add one vWidth line
	ld b, a							;store it
	sub 50							;subtract row 56
	jr z, dispcol2					;if zero, we are at bottom of screen
	jp p, dispcol2					;if positive, we are at bottom of screen
	ld a, b							;recall new row value
	ld (_penRow), a					;write it back
	ld a, (pagelist_entries)
	ld b, a
	ld a, (listcounter)
	cp b
	ret z							;if they match, we are done with displaying name lists
	ld l, a							;loading listcounter into hl
	ld h, 0
	call _mulHL10					;multiply by ten
	ld de, pagelist
	add hl, de						;hl points to entry to display
	ld de, var_name
	ld bc, $000A
	ldir							;copy it to temp location
	ld a, (selected)
	ld b, a
	ld a, (listcounter)
	cp b
	call z, inverse_on				;turn on inverse text if we are displaying the selected item
	ld hl, var_name+1				;skip first byte, which is a type byte.  Get length byte.
	ld b, (hl)						;get the length byte into b
	inc hl							;point to first char in name
	ld a, (_pencol)
	push af							;save column coord
disploop1_chars:
	ld a, (hl)						;get character to print
	push hl							;preserve pointer
	push bc							;preserve counter
	call _vputmap					;display char
	pop bc
	pop hl
	inc hl							;point to next char
	djnz disploop1_chars			;do until done with that name
	pop af
	ld (_pencol), a					;recall column coords
	ld a, (selected)
	ld b, a
	ld a, (listcounter)
	cp b
	call z, inverse_off
	inc a
	ld (listcounter), a				;update the list progress counter
	jr disploop1					;reloop for the other names

dispcol2:							;display column 2
	ld hl, $0329
	ld (_pencol), hl
disploop2:							;handles column 2
	ld a, (_penRow)					;current row coord
	add a, $06						;add one vWidth line
	ld b, a							;store it
	sub 50							;subtract row 56
	jr z, dispcol3					;if zero, we are at bottom of screen
	jp p, dispcol3					;if positive, we are at bottom of screen
	ld a, b							;recall new row value
	ld (_penRow), a					;write it back
	ld a, (pagelist_entries)
	ld b, a
	ld a, (listcounter)
	cp b
	ret z							;if they match, we are done with displaying name lists
	ld l, a
	ld h, 0
	call _mulHL10					;multiply by ten
	ld de, pagelist
	add hl, de						;hl points to entry to display
	ld de, var_name
	ld bc, $000A
	ldir							;copy it to temp location
	ld a, (selected)
	ld b, a
	ld a, (listcounter)
	cp b							;displaying selection?  Then invert
	call z, inverse_on
	ld hl, var_name+1				;skip first byte, which is a type byte.  Get length byte.
	ld b, (hl)						;get the length byte into b
	inc hl							;point to first char in name
	ld a, (_pencol)
	push af							;save column coord
disploop2_chars:
	ld a, (hl)
	push hl
	push bc
	call _vputmap					;display char
	pop bc
	pop hl
	inc hl							;point to next char
	djnz disploop2_chars			;do until done with that name
	pop af
	ld (_pencol), a					;recall column coords
	ld a, (selected)
	ld b, a
	ld a, (listcounter)
	cp b
	call z, inverse_off
	inc a
	ld (listcounter), a				;update the list progress counter
	jr disploop2					;reloop for the other names

dispcol3:
	ld hl, $0351
	ld (_pencol), hl
disploop3:							;handles column 2
	ld a, (_penRow)					;current row coord
	add a, $06						;add one vWidth line
	ld b, a							;store it
	sub 50							;subtract row 56
	ret z							;if zero, we are at bottom of screen, and 3 columns is the max
	ret p							;if positive, we are at bottom of screen, and 3 columns is the max
	ld a, b							;recall new row value
	ld (_penRow), a					;write it back
	ld a, (pagelist_entries)
	ld b, a
	ld a, (listcounter)
	cp b
	ret z							;if they match, we are done with displaying name lists
	ld l, a
	ld h, 0
	call _mulHL10					;multiply by ten
	ld de, pagelist
	add hl, de						;hl points to entry to display
	ld de, var_name
	ld bc, $000A
	ldir							;copy it to temp location
	ld a, (selected)
	ld b, a
	ld a, (listcounter)
	cp b							;are we displaying the selected item?
	call z, inverse_on				;turn on inverted text
	ld hl, var_name+1				;skip first byte, which is a type byte.  Get length byte.
	ld b, (hl)						;get the length byte into b
	inc hl							;point to first char in name
	ld a, (_pencol)
	push af							;save column coord
disploop3_chars:
	ld a, (hl)
	push hl
	push bc
	call _vputmap					;display char
	pop bc
	pop hl
	inc hl							;point to next char
	djnz disploop3_chars			;do until done with that name
	pop af
	ld (_pencol), a					;recall column coords
	ld a, (selected)
	ld b, a
	ld a, (listcounter)
	cp b
	call z, inverse_off				;turn off inverted text
	inc a
	ld (listcounter), a				;update the list progress counter
	jr disploop3					;reloop for the other names

inverse_on:							;turns on inverse text
	set textInverse, (IY+textFlags)
	ret
inverse_off:
	res textInverse, (IY+textFlags)
	ret



highlight:
	ld hl, $FC90					;row nine
	ld de, $0060					;six rows to add
	ld a, (selected)				;current selection
	push af
	sub 7
	jp p, highlight_col2			;if over 7, then move over a column first
	pop af
findloop:
	cp 0							;done?
	jr z, makeblock					;the correct point has been found.  Now draw a block there
	add hl, de						;add six lines
	dec a							;decrement counter
	jr findloop						;reloop
highlight_col2:
	inc hl
	inc hl
	inc hl
	inc hl							;offset hl to draw in column two
	inc hl
	pop af							;get selected item number
	push af
	sub 14
	jp p, highlight_col3			;if greater than 20, it is in column 3
	pop af
	sub 7							;start at top of column
	jr findloop
highlight_col3:
	inc hl
	inc hl
	inc hl
	inc hl							;offset hl to draw in the third column
	inc hl
	pop af
	sub 14							;start at top of column
	jr findloop
makeblock:	
	ld b, 6
makeblock_loop:						;makes a black block 6X32 pixels
	push hl
	ld (hl), $FF
	inc hl
	ld (hl), $FF
	inc hl
	ld (hl), $FF
	inc hl
	ld (hl), $FF
	inc hl
	ld (hl), %11110000
	pop hl
	ld de, $0010					;one row
	add hl, de
	djnz makeblock_loop				;loop until whole block drawn
	ret



feedback:					;displays user feedback info
	ld hl, $FF80			;start byte of row 56 in video mem
	ld b, 16				;initialize counter
HLineLoop:
	ld (hl), $FF				;darken 8 pixels
	inc hl					;next byte
	djnz HLineLoop			;recycle
	ld hl, $3A00
	ld (_pencol), hl
	ld hl, showtext
	call _vputs					;'Show: '
	ld a, (filter_type)			;determine what var type is being shown
	and %00000001				;bit 0 counts.  Is it odd or even?
	cp 0
	jr nz, show_odd
	ld hl, PRGMtext
	jr showfilter
show_odd:
	ld hl, STRNtext
showfilter:
	call _vputs					;'PRGM' or 'STRN'
	ld hl, $3A5A
	ld (_pencol), hl
	ld hl, pagetext
	call _vputs					;'Page: '
	ld a, (cur_page)
	inc a						;internally, page = 0 and up; display as 1 and up
	add a, L0					;make it an ASCII char for that number
	call _vputmap				;display it
	ld a, ' '
	call _vputmap
	ld a, 'o'
	call _vputmap
	ld a, 'f'
	call _vputmap
	ld a, ' '
	call _vputmap
	ld a, (num_pages)
	inc a
	add a, L0
	call _vputmap				;now reads, 'Page: 1 of 1' if it is page 1, out of 1 pages.
	ret


keyloop:							;input loop	
	call _get_key					;get a keypress
	cp 0
	jr z, keyloop					;if no key is pressed, reloop
	cp k_up
	jp z, up						;UP = move selection up
	cp k_down
	jp z, down						;DOWN = move selection down
	cp k_left
	jp z, left
	cp k_right
	jp z, right
	cp k_exit
	jp z, quit						;exit key = quit
	cp k_enter
	jp z, menu						;ENTER = activate menu
	cp k_del
	jp z, main_delete				;can't go directly to delete routine
	cp k_second
	jp z, run						;2ND = run program
	cp k_graph
	jp z, options					;GRAPH = Options menu
	cp k_xvar
	jp z, toggle_filter				;X-VAR = change filter type
	cp k_prgm
	jp z, lock						;PRGM = Lock a BASIC program
	cp k_alpha
	jp z, menu						;ALPHA = Activate Actions menu
	cp k_more
	jp z, nextpage					;MORE = view next screen page
	cp k_log
	jp z, power_down				;LOG = Turn off the calc (remains in shell when turned on)
	cp k_plus
	jr z, main_inc_contrast			;+ = Increase contrast
	cp k_minus
	jr z, main_dec_contrast			;- = Decrease contrast
	sub $30							;leaves only Fkeys, second, exit, and more as nonzero positive
	jr z, keyloop					;if it was $30, we don't want it
	jp m, keyloop					;if it was less than $30, we don't want it
	sub 6							;leaves second as zero, and exit and more as positive
	jr z, keyloop
	jp p, keyloop					
	jp menu							;ANY F-KEY = activate menu 
main_delete:						;can't go direct to delete routine from mainscreen without this stuff
	call select						;determine name of selected var
	call makeWindow					;create window
	jp delete						;delete routine as normal
main_inc_contrast:
	call inc_contrast
	jr keyloop
main_dec_contrast:
	call dec_contrast
	jr keyloop


up:									;move selection up
	ld a, (pagelist_entries)
	cp $FF
	jr z, keyloop					;if there are no progs, stop
	ld a, (selected)				;get current highlighted item
	cp 0
	jp z, keyloop					;if already at the top, don't do anything
	dec a
	ld (selected), a				;new selection value
	call display					;redraw the display
	jp keyloop						;back to user input

down:
	ld a, (pagelist_entries)
	cp $FF
	jp z, keyloop					;if there are no progs, stop
	ld b, a
	ld a, (selected)
	inc a							;the count starts at zero instead of one, so inc it
	cp b
	jp z, keyloop					;don't exceed the number of vatlist entries
	cp 21		;*****				;max value for the page
	jp z, keyloop					;can't go off the page
	ld (selected), a				;new selection value (already incremented)
	call display					;redraw
	jp keyloop						;return to user input loop

left:
	ld a, (pagelist_entries)
	cp $FF
	jp z, keyloop					;stop if there are no valid progs
	ld a, (selected)				;get current selection
	sub 7							;subtracting seven moves left one column
	jp m, keyloop					;if the number is negative, it goes off the screen
	ld (selected), a				;load new value to selection counter
	call display					;redraw
	jp keyloop						;return to user input loop

right:
	ld a, (pagelist_entries)
	cp $FF
	jp z, keyloop					;stop if there are no valid progs
	ld a, (selected)				;get current selection
	add a, 7						;add 7 to move right one column
	ld b, a							;hold that number...
	sub 21							;21 is highest allowable selection number on the page
	jp p, keyloop					;if it is positive, a move to the right goes off the page
	ld a, (pagelist_entries)
	ld c, a
	ld a, b							;the selection number + 7 that we stored earlier
	sub c							;subtract the number of programs from the new selection value
	jp p, keyloop					;if positive, the selection is higher than the number of valid programs
	ld a, b							;recall it again
	ld (selected), a				;save it
	call display					;redraw
	jp keyloop						;return to user input loop


select:								;determines what var was selected.  Puts name in var_name.
	ld a, (selected)
	ld b, a
	ld a, (pagelist_entries)
	cp $FF
	jp z, keyloop					;stop if there are no programs
	sub b							;entries-selection
	jr z, select_continue			;if zero, they match
	jp m, keyloop					;if negative, selection does not exist.  If positive, it does.
select_continue:
	ld l, b
	ld h, 0
	call _mulHL10					;HL*10
	ld de, pagelist
	add hl, de						;hl now points to entry of selected program
	ld de, var_name
	ld bc, $000A
	ldir							;copy the entry to var_name
	ld hl, var_name					;point to the name
	ret


menu:	
	call select						;determines what var was selected, puts name in var_name.
	call makeWindow					;makes a window in center of screen
	ld hl, $0F1A
	ld (_pencol), hl
	ld hl, vartext
	call _vputs						;'Variable: '
	call display_varname			;displays variable name in quotes	
	ld hl, $181F
	ld (_pencol), hl
	ld hl, action_menutable
	call makemenu
	jr nz, menu_cancel
	cp 1
	jp z, menu_run
	cp 2
	jp z, copy
	cp 3
	jp z, rename
	cp 4
	jp z, delete
	cp 5
	jp z, convert
	cp 6
	jp z, info
menu_cancel:						;if exit was pressed
	call destroyWindow				;erases window and replaces background
	jp keyloop						;back to user input if user canceled lock procedure
menu_run:
	call destroyWindow
	ld hl, var_name					;needed by run routine
	jp run_bypass_select			;pre-selected, so skip selection call in run routine


makeWindow:							;creates a window in the center of the screen
	ld hl, $FC00					;beginning of video memory. Save the screen.
	ld de, _plotSScreen				;graph screen.  I'm going to copy a pic of the screen here.
	ld bc, 1024						;length of vidmem
	ldir							;block copy
	set graphdraw,(IY+graphflags)	;this will clean up the graph screen next time GRAPH is pushed in TI-OS.
clrWindow:							;call this if you just want to clean up the window
	ld hl, $FCC3-1
	ld a, (hl)
	and %11111100
	ld (hl), a
	inc hl
	push hl
	ld b, 10
	ld a, 55
	ld de, $0010
mw_clearloop:
	ld (hl), $00
	inc hl
	djnz mw_clearloop				;clear [b] bytes (one line)
	pop hl
	add hl, de
	push hl
	ld b, 10
	dec a
	cp 0
	jr nz, mw_clearloop				;clear [a] lines
	pop hl
	ld hl, $FCD3
	push hl
	ld de, $0010
	ld b, 10
	ld c, 2
	dec hl
	ld a, (hl)
	and %11111100
	ld (hl), a
	inc hl
mw_topbar:
	ld (hl), $FF					;making a short black line
	inc hl
	djnz mw_topbar					;fill [b] bytes on a line
	ld a, (hl)
	and %00111111
	ld (hl), a
	pop hl
	add hl, de
	push hl
	dec hl
	ld a, (hl)
	and %11111100
	ld (hl), a
	inc hl
	ld b, 10
	dec c
	ld a, c
	cp 0							;we will make it two layers
	jr nz, mw_topbar				;reloop for two lines thick
	pop hl
	ld b, 48
	push hl
mw_sidesloop:
	dec hl
	ld a, (hl)
	and %11111100
	ld (hl), a
	inc hl
	ld (hl), %10000000				;draw the left side of the window
	ld a, 9
	add a, l
	ld l, a
	ld (hl), %00000001
	inc hl
	ld a, (hl)
	and %00111111
	ld (hl), a
	pop hl
	add hl, de
	push hl
	djnz mw_sidesloop
;******
	ld b, 10
	ld hl, $FD63
	call mw_hlineloop				;draw horizontal line to underline window title text
;******
	pop hl
	ld b, 10
	dec hl
	ld a, (hl)
	and %11111100
	ld (hl), a
	inc hl
mw_hlineloop:						;draws a horizontal line [b] bytes wide
	ld (hl), $FF					;draw a line for the bottom
	inc hl
	djnz mw_hlineloop
	ld a, (hl)
	and %00111111
	ld (hl), a
	ret

destroyWindow:						;replaces the window with the saved screen pic
	ld hl, _plotSScreen
	ld de, $FC00
	ld bc, 1024
	ldir
	ret
	


display_varname:					;displays variable name in quotes
	ld a, 34						;charcode for '"' (quote)
	call _vputmap
	ld hl, var_name+1
	ld b, (hl)						;name length
	inc hl
dv_nameloop:					;display the name of the selected program
	ld a, (hl)						;get a character
	push bc
	push hl
	call _vputmap					;display character
	pop hl
	pop bc
	inc hl
	djnz dv_nameloop			;loop until name is displayed
	ld a, 34
	call _vputmap					;disp a quotemark
	ret





toggle_filter:						;toggles between prgm and string view modes
	ld a, (filter_type)				;get current type
	inc a							;if odd, make it even.  If even, make it odd.
	ld (filter_type), a				;reload it
	xor a
	ld (cur_page), a				;set to first page
	jp top							;search and redraw



nextpage:							;advances screen page, or recycles to page 0 if at the last page
	ld a, (num_pages)				;total pages
	ld b, a
	ld a, (cur_page)				;current page
	cp b							;are we on the last page right now?
	jr z, nextpage_recycle			;if so, we will recycle to page 0
	cp 8							;is this the last allowable page?
	jr z, nextpage_recycle
	inc a
	ld (cur_page), a				;otherwise, increment the page number
	xor a
	ld (selected), a
	call pagify						;create pagelist for new page
	call display
	jp keyloop
nextpage_recycle:
	cp 0							;already on page zero?
	jp z, keyloop					;if so, no need to redraw
	xor a
	ld (cur_page), a				;page 0
	ld (selected), a				;set selection to zero
	call pagify						;create pagelist for page zero
	call display					;redraw
	jp keyloop						;redraw without a re-build of the variable list



fetch_header:							;finds var at ahl, stores length header and first 2 bytes of data
	call decodeABS						;decode the absolute address. a=RAM page, hl=offset into swap area
	add hl, de							;add $4000 to hl, to make it a valid pointer into first swap area
	add a, 64							;set bit 6, so it represents a RAM page and not ROM
	out (5), a							;load RAM page to swapping area $4000-$7FFF
	inc a								;next page
	out (6), a							;load next RAM page to swapping area $8000-$BFFF for continuity
	ld a, (hl)
	ld (var_length), a
	inc hl
	ld a, (hl)
	ld (var_length+1), a
	inc hl
	ld a, (hl)
	ld (var_firstbyte), a
	inc hl
	ld a, (hl)
	ld (var_secondbyte), a	
	ld a, $0D							;ROM page containing call table
	out (5), a							;put it back so that I can use ROM calls again
	ret									;done



copy:									;copies a variable
	call clrWindow
	ld hl, $0F1A
	ld (_pencol), hl
	ld hl, copytext
	call _vputs							;'COPY'
	ld a, (_pencol)
	add a, 6
	ld (_pencol), a
	call display_varname				;display the selected variable's name
	call copy_core						;copy and rename do essentially the same thing, so reuse the code
	jp z, top							;if nz, there was an error copying
copy_cancel:
	call destroyWindow
	jp keyloop


rename:
	ld hl, var_name+1					;skip type byte
	ld de, launcher_name				;we need to compare to launcher name.  If same, abort.
	ld b, 8
	call string_compare
	jr z, ren_cancel					;don't let user rename the launcher
	ld hl, var_name+1					;skip type byte
	ld de, self_name					;we need to compare to this prog's name.  If same, abort.
	ld b, 8
	call string_compare
	jr z, ren_cancel					;don't let user rename the shell program
	call clrWindow						;clear the window
	ld hl, $0F1A
	ld (_pencol), hl
	ld hl, rentext
	call _vputs							;'RENAME'
	ld a, (_pencol)
	add a, 6
	ld (_pencol), a
	call display_varname				;display the selected variable's name
	call copy_core						;copy and rename do essentially the same thing, so reuse the code
	jr nz, ren_cancel					;if nz, there was an error copying
	ld hl, var_name						;old variable name
	rst 20h								;put it in OP1
	rst 10h								;_findsym
	call _delvar						;delete the old variable
	jp top								;rebuild list
ren_cancel:
	call destroyWindow
	jp keyloop


copy_core:								;copies the var to new var
	ld hl, $171A
	ld (_pencol), hl
	ld hl, totext
	call _vputs							;'to:'
	ld hl, $201F
	ld (_pencol), hl
	ld a, 5
	call _vputmap						;display a little arrow
	ld hl, $0506
	ld (_currow), hl
	set 2, (IY+$1E)						;shows a shift-lock (2nd/alpha/lowercase) indicator in upper right
	set shiftAlpha, (IY+shiftFlags)		;set to alpha initially
	set shiftALock, (IY+shiftFlags)		;set to Alpha Lock
	ld b, 8								;we will accept 8 characters max
	ld hl, new_name+2					;place to store textIn's string to
	call textIn							;get up to 8 chars of text from user for new var name
	res 2, (IY+$1E)
	res shiftAlpha, (IY+shiftFlags)	
	jp nz, copy_error					;if nz, exit was pressed
	ld a, 8
	sub b								;see how many chars were entered
	cp 0
	jp z, copy_error					;disallow zero-character names
	ld (new_name+1), a					;load the length to the proper place in the string
	ld hl, new_name
	rst 20h
	rst 10h
	jr c, copy_continue					;if carry, does not previously exist
	ld hl, var_name+1
	ld de, new_name+1
	ld a, (new_name+1)
	ld b, a								;comparing new and old for b bytes
	call string_compare					;to make sure we aren't renaming it to itself
	jp z, copy_error					;can't rename to itself
	call clrWindow
	ld hl, $0F1A
	ld (_pencol), hl
	ld hl, overwritetext
	call _vputs							;'Overwrite?'
	ld hl, $201F
	ld (_pencol), hl
	ld hl, f1text
	call _vputs							;'F1 - '
	ld hl, confirmtext
	call _vputs							;'CONFIRM'	
	call _getkey
	cp kf1
	jr nz, copy_error					;if nz, user canceled
copy_continue:
	ld hl, var_name
	rst 20h
	rst 10h
	ld a, (hl)							;get type byte from VAT entry
	push af								;store the type byte
	ld a, b
	ex de, hl							;ld ahl, bde
	push af
	push hl								;store ahl address of variable
	call fetch_header					;get length
	ld hl, new_name
	rst 20h								;put new name in OP1
	ld hl, (var_length)					;get the length of the var into hl
	call _createprog					;create a program var
	pop hl
	pop af								;recall ahl address
	call $4c3f							;AHL+2 to get around the length bytes
	call _set_abs_src_addr				;we will copy from here
	ld hl, new_name
	rst 20h
	rst 10h
	pop af								;recall type byte
	ld (hl), a							;put the old type byte in the new variable to make it the same type
	ld a, b
	ex de, hl							;ld ahl, bde
	call $4c3f							;skip length bytes
	call _set_abs_dest_addr				;we will copy to here
	ld hl, (var_length)
	xor a
	call _set_mm_num_bytes				;set the copy length
	call _mm_ldir						;fill the new var with the old var's data
	cp a								;set the zero flag = return without error
	ret
copy_error:
	xor a
	cp 1								;res the zero flag = return with error
	ret
	


delete:
	call clrWindow						;clear the window
	ld hl, $0F1A
	ld (_pencol), hl
	ld hl, deltext
	call _vputs							;'DELETE'
	ld a, (_pencol)
	add a, 6							;move over 6 pixels
	ld (_pencol), a
	call display_varname				;display the name of the selected variable in quotes
	ld hl, $201F
	ld (_pencol), hl
	ld hl, f1text
	call _vputs							;'F1 - '
	ld hl, confirmtext
	call _vputs							;'CONFIRM'
	call _getkey
	cp kf1
	jr z, del_continue					;if F1, then it was confirmed
	call destroyWindow					;remove the window and replace background if it wasn't confirmed
	jp keyloop							;return to user input
del_continue:							;confirmed. Now delete it.
	ld hl, var_name						;point to the name of the variable
	rst 20h								;move to OP1
	rst 10h								;_findsym
	call _delvar						;delete it
	jp top								;rebuild the list



convert:
	ld hl, var_name+1					;skip type byte
	ld de, self_name					;we need to compare to this prog's name.  If same, abort.
	ld b, 8
	call string_compare
	jr z, conv_cancel					;don't let user convert the shell program
	call clrWindow						;clear the window contents
	ld hl, $0F1A
	ld (_pencol), hl
	ld hl, convtext
	call _vputs							;'CONVERT'
	ld a, (_pencol)
	add a, 6
	ld (_pencol), a
	call display_varname				;display the name of selected variable
	ld hl, $171A
	ld (_pencol), hl
	ld hl, totext
	call _vputs							;'to:'
	ld hl, $201F
	ld (_pencol), hl
	ld hl, convert_menutable
	call makemenu
	jr nz, conv_cancel
	cp 1
	jr z, conv_prgm
	cp 2
	jr z, conv_strn
conv_cancel:
	call destroyWindow					;remove the window and replace background
	jp keyloop							;back to user input
conv_prgm:
	ld hl, var_name
	rst 20h								;move to OP1
	rst 10h								;_findsym
	ld (hl), $12						;load the PRGM type byte to the VAT entry
	jp top								;rebuild the list
conv_strn:
	ld hl, var_name
	rst 20h								;move to OP1
	rst 10h								;_findsym
	ld (hl), $0C						;load a STRN type byte to the VAT entry
	jp top								;rebuild the list
	


info:
	call clrWindow
	ld hl, $0F1A
	ld (_pencol), hl
	ld hl, infotext
	call _vputs							;'INFO'
	ld a, ':'
	call _vputmap						;':'
	call display_varname				;display variable name
	ld hl, $201A
	ld (_pencol), hl
	ld hl, sizetext
	call _vputs							;'Size: '
	ld hl, var_name
	rst 20h
	rst 10h
	ld a, b
	ex de, hl							;ld ahl, bde
	call fetch_header					;get the size and first two bytes of variable
	ld hl, (var_length)
	ld de, 8							;add 8 bytes for the VAT entry and length bytes, so it agrees with TI-OS's value
	add hl, de
	ld a, (var_name+1)					;name length
	ld e, a
	add hl, de							;add the length of the VAT name so it agrees with TI-OS's length value
	call DisplayHL							;display HL in vwidth chars at pen location
	ld a, ' '
	call _vputmap
	ld hl, bytestext
	call _vputs							;'bytes'
	ld hl, $261A
	ld (_pencol), hl
	ld hl, typetext
	call _vputs							;'Type: '
	ld hl, var_name
	rst 20h
	rst 10h
	ld a, (hl)							;get type byte from VAT entry
	cp $0C								;chech if it is a string
	jr z, info_strn						;disp string info.  Otherwise, it is a prgm...
info_prgm:
	ld a, (var_firstbyte)				;get first byte of program data
	cp $8E
	jr z, info_8e
info_basic:
	ld hl, basictext
	call _vputs							;'BASIC '
	ld hl, prgmtext
	call _vputs							;'PRGM'
	jr info_done
info_8e:								;first byte is $8E
	ld a, (var_secondbyte)
	cp $28
	jr nz, info_basic					;$8E $28 = compiled assembly program
	ld hl, asmtext
	call _vputs
	ld hl, prgmtext
	call _vputs
	jr info_done
info_strn:
	ld hl, STRNtext
	call _vputs							;'STRN'
info_done:
	call _get_key						;wait for a keypress
	cp 0
	jr z, info_done
	call destroyWindow					;remove the window and replace background
	jp keyloop							;return to user input loop





run:									;runs a program variable
	call select							;get name of selected variable
run_bypass_select:						;if RUN selected from menu, must enter routine from here
	ld hl, var_name+1					;skip type byte
	ld de, launcher_name				;we need to compare to launcher name.  If same, abort.
	ld b, 8
	call string_compare
	jp z, keyloop						;don't let user run the launcher
	ld hl, var_name+1
	ld de, self_name
	ld b, 8
	call string_compare					;compare to self name
	jp z, keyloop						;don't let user run this program
	ld hl, flagbyte
	bit anticrash, (hl)					;check whether anticrash is active
	call nz, set_onChecksum				;sets correct page2 power-up checksum
	ld hl, var_name						;point to selected variable's name
	rst 20h								;hl points to var_name.  Move it to OP1
	rst 10h
	ld a, (hl)							;get the type byte from the VAT entry
	cp $12								;$12 denotes a program
	jp nz, run_error					;uh-oh... not a program
	ld a, b
	ex de, hl							;ld ahl, bde
	call fetch_header					;get length and first two data bytes
	ld hl, var_name
	rst 20h								;put name into OP1
	ld a, (var_firstbyte)
	cp  $8E
	jr nz, run_basic					;if not $8E, then it is a basic program
	ld a, (var_secondbyte)
	cp $28
	jr z, run_asm						;8E29=compiled ASM
	cp $27
	jr nz, run_basic					;8E27=tokenized uncompiled ASM
run_asm:								;run as an assembly program
	call _exec_assembly					;name is in OP1
	jp progstart						;all our dynamic data is probably trashed, so rebuild everthing
run_basic:								;run as a basic program
	res 4, (IY+$09)						;tell TI-OS to ignore the on key if it was pressed
	ld hl, var_name+1					;name length
	ld c, (hl)
	ld a, (hl)
	add a, $32							;add the name-length preceder token
	ld (launcher_code_execname), a		;correct it for proper name length
	ld de, launcher_code_execname+1
	ld hl, var_name+2					;point to name of program to run
	ld b, 0								;bc = length
	ldir								;name and preceder token now in launcher code
	ld hl, launcher_name-1
	rst 20h								;put name in OP1
	rst 10h
	ld a, b
	ex de, hl
	call decodeABS						;decode abs address of variable data
	add a, 64							;set bit 7, this is a RAM page
	out (5), a
	inc a
	out (6), a							;load RAM pages that variable resides in
	ld de, $4002						;+2 skips length bytes
	add hl, de							;add offset into first page swap space, and up to the name string in the launcher
	ex de, hl							;put it in de
	ld hl, launcher_code				;read from here
	ld bc, launcher_code_end-launcher_code	;length of launcher program to copy
	ldir								;copy into the program
	ld a, $0D
	out (5), a							;reload ROM page with call table on it
	call _clrLCD
	call _runindicon					;turn on run indicator
	ld hl, var_name
	rst 20h
	rst 10h
	ld a, b
	ex de, hl
	call fetch_header
	ld a, (var_firstbyte)
	cp 0
	call z, r_b_tok						;if untokenized, display a message saying it is being tokenized
	call _op1set1						;set OP1 to 1
	call _stoans						;set Ans = OP1, so launcher will know we want to run BASIC program
	ret									;quit, drop back to (new) launcher, run BASIC program.
r_b_tok:
	call makeWindow						;make a window
	ld hl, $0F1A
	ld (_pencol), hl
	ld hl, runtext
	call _vputs							;'RUN '
	call display_varname				;display the variable name
	ld hl, $1F1A
	ld (_pencol), hl
	ld hl, toktext
	call _vputs							;'Tokenizing...'
	ret
run_error:								;if it wasn't a program
	call makeWindow						;make the window
	ld hl, $0F1A
	ld (_pencol), hl
	ld hl, runtext
	call _vputs							;'RUN'
	call display_varname				;disp the var name
	ld hl, $1F1A
	ld (_pencol), hl
	ld hl, errortext
	call _vputs							;'ERROR - '
	ld hl, notprgmtext
	call _vputs							;'NOT A PROGRAM'
	call _getkey						;wait for a keypress
	call destroyWindow					;remove the window
	jp keyloop							;user input loop



lock:									;lock a BASIC program
	call makeWindow
	ld hl, $0F1A
	ld (_pencol), hl
	ld hl, locktext
	call _vputs							;'LOCK '
	call select							;identify selected program
	call display_varname				;display the variable name
	ld hl, $1F1A
	ld (_pencol), hl					;set pen coords
	ld hl, var_name
	rst 20h
	rst 10h
	ld a, (hl)							;get type byte
	cp $12
	jr z, lock_prgm
	ld hl, errortext
	call _vputs							;'ERROR - '
	ld hl, notprgmtext
	call _vputs							;'NOT A PRGM'
	call _getkey						;wait for keypress
	call destroyWindow
	jp keyloop
lock_prgm:
	ld a, b
	ex de, hl							;ld ahl, bde
	call fetch_header					;get length and first two data bytes
	ld a, (var_firstbyte)
	cp 0
	jr z, lock_text						;plaintext basic
	cp $8E
	jr nz, lock_valid					;tokenized basic
	ld a, (var_secondbyte)
	cp $29
	jr z, lock_error_locked				;already locked (tokenized basic)
	cp $28
	jr z, lock_error_asm				;assembly program (compiled)
	cp $27
	jr z, lock_error_asm				;assembly program (uncompiled tokenized)
lock_valid:								;valid unlocked basic
	ld hl, $1F1F
	ld (_pencol), hl
	ld hl, f1text
	call _vputs							;'F1 - '
	ld hl, confirmtext
	call _vputs							;'CONFIRM'
lock_keyloop:	
	call _getkey
	cp kf1
	jr nz, lock_cancel					;if it was not confirmed
	call editlock_core					;the edit-locking routine
	jp top								;rebuild the list
lock_text:								;text basic
	ld a, (var_secondbyte)
	cp $00
	jr nz, lock_valid					;valid text program
lock_error_locked:						;already locked
	ld hl, errortext
	call _vputs							;'ERROR - '
	ld hl, $251A
	ld (_pencol), hl
	ld hl, lockedtext
	call _vputs							;'ALREADY EDIT-LOCKED'
	call _getkey
	call destroyWindow
	jp keyloop
lock_error_asm:							;assembly program
	ld hl, errortext
	call _vputs							;'ERROR - '
	ld hl, $251A
	ld (_pencol), hl
	ld hl, notbasictext
	call _vputs							;'NOT A BASIC PRGM'
	call _getkey
lock_cancel:							;jp'd to if canceled
	call destroyWindow
	jp keyloop

editlock_core:							;core code of the edit-lock routine
	ld hl, temp_name-1
	rst 20h								;put temp name in OP1
	ld hl, (var_length)					;get original prgm length
	ld a, (var_firstbyte)				;first data byte of program
	cp 0
	jr z, lock_1addone					;if first byte=0, editlocking will add one byte
	inc hl								;if not, we will add two...
lock_1addone:
	inc hl								;we now have the proper length for the temp program
	call _createprog					;allocate hl bytes for temp program with name in OP1
	ld a, b
	ex de, hl							;ld ahl, bde
	call $4C3F							;increment AHL twice to skip length bytes
	push af								;save a from ahl
	ld a, (var_firstbyte)
	cp 0
	jr z, lock_2addone					;if 0, then increment only once
	pop af
	call _inc_ptr_ahl
	push af
lock_2addone:
	pop af
	call _inc_ptr_ahl						;we have now incremented into the dest. program far enough
	call _set_abs_dest_addr				;we will copy to the temp program
	ld hl, var_name
	rst 20h								;move name to OP1
	rst 10h								;_findsym
	ld a, b
	ex de, hl							;ld ahl, bde
	call $4C3F							;skip length bytes
	call _set_abs_src_addr				;we will copy from here
	xor a								;zero out the a in ahl
	ld hl, (var_length)					;original length
	call _set_mm_num_bytes				;copy length
	call _mm_ldir						;block copy
	ld hl, temp_name-1
	rst 20h								;move name to OP1
	rst 10h								;_findsym
	ld a, b
	ex de, hl							;ld ahl, bde
	call decodeABS						;decode the abs address. a=page hl=offset
	ld de, $4000
	add hl, de							;point into first swap area
	add a, 64							;set bit 6, it is a RAM page
	out (5), a							;swap in a RAM page
	inc a
	out (6), a							;swap in next RAM page
	ld a, (hl)
	ld (var_length), a					;new program's length
	inc hl
	ld a, (hl)
	ld (var_length+1), a				;we now have both bytes saved
	inc hl								;points to first data byte of new prog, which we will modify
	ld a, (var_firstbyte)				;first byte of original prog
	cp 0
	jr nz, tokenized					;if starts with 0, then it is plaintext, otherwise tokenized
plaintext:								;set it to 'plaintext, edit-locked' ($00 $00)
	ld (hl), a							;set first byte to zero.  Second byte is already a zero.  Done edit-locking.
	jr editlocked
tokenized:								;set it to 'tokenized, edit-locked' ($8e $29)
	ld (hl), $8E
	inc hl
	ld (hl), $29
editlocked:								;editlocking is done.  Now copy back to original prgm name.
	ld a, $0d
	out (5), a							;put ROM page 0D back so we can make ROM calls
	ld hl, var_name					;the original program
	rst 20h								;move name to OP1
	rst 10h								;_findsym
	call _delvar						;delete the old program var
	ld hl, var_name
	rst 20h								;put the name back in OP1
	ld hl, (var_length)					;get new program length
	call _createprog					;make a new program under original name.
	ld a, b
	ex de, hl							;ld ahl, bde
	call $4C3F							;AHL+2, get past length bytes
	call _set_abs_dest_addr				;we're gonna put the old data and edit-lock codes in to the new program
	ld hl, temp_name-1
	rst 20h								;move to OP1
	rst 10h								;_findsym
	ld a, b
	ex de, hl							;ld ahl, bde
	call $4C3F							;skip length bytes
	call _set_abs_src_addr				;get the old data location (in the temp var)
	xor a
	ld hl, (var_length)					;get the length to copy
	call _set_mm_num_bytes				;set it
	call _mm_ldir						;copy
	ld hl, temp_name-1
	rst 20h								;move name to OP1
	rst 10h								;_findsym
	call _delvar						;delete the temporary program var
	ret


power_down:								;turn off the calc
	call set_onChecksum					;set correct checksum so calc doesn't think it crashed if you pull the batts
	ld a, 1
	out (3), a							;turn off screen and timer interrupts
	ei									;enable interrupts
	halt								;shut down and wait for ON key
	ld a, 11
	out (3), a							;turn on screen and timer interrupts
	res 4, (IY+9)						;'onInterrupt, iy+onflags'  so calc won't see the on key
	jp keyloop							;return to user input loop



options:								;Option menu
	call makeWindow						;create a window
options_bypass:							;to go to it without creating a window
	call clrWindow
	ld hl, $0F1A
	ld (_pencol), hl
	ld hl, optionstext
	call _vputs							;'Options'
	ld hl, $244A
	ld (_pencol), hl

	ld hl, flagbyte						;point to the flag byte
	bit anticrash, (hl)					;check whether crash protection is on or not
	jr z, options_ac_statOff
options_ac_statOn:
	ld hl, ontext
	call _vputs							;'ON  '
	jr options_ac_statdone
options_ac_statOff:
	ld hl, offtext
	call _vputs							;'OFF'
options_ac_statdone:
	ld hl, $181F
	ld (_pencol), hl
	ld hl, option_menutable
	call makemenu
	jr nz, options_cancel
	cp 1
	jr z, options_shutdown
	cp 2
	jr z, options_contrast
	cp 3
	jr z, options_anticrash
options_cancel:
	call destroyWindow
	jp keyloop
options_contrast:
	ld hl, $302A
	ld (_pencol), hl
	ld a, 6								;up arrow
	call _vputmap
	ld hl, $362A
	ld (_pencol), hl
	ld a, 7								;down arrow
	call _vputmap
	ld hl, $3231
	ld (_pencol), hl
	ld hl, adjusttext
	call _vputs							;'Adjust'
options_contrast_keyloop:
	call _get_key
	cp 0
	jr z, options_contrast_keyloop
	cp k_up
	jr z, options_contrast_up
	cp k_down
	jr nz, options_bypass				;if not valid key, assume user canceled
options_contrast_down:
	call dec_contrast					;decrement contrast value
	jr options_contrast_keyloop			;reloop
options_contrast_up:
	call inc_contrast
	jr options_contrast_keyloop
options_anticrash:
	ld hl, $244A
	ld (_pencol), hl
	ld b, 12
options_anticrash_clrLoop:
	ld a, ' '
	call _vputmap
	djnz options_anticrash_clrLoop
	ld hl, $244B
	ld (_pencol), hl
	ld hl, anticrash_menutable
	call makemenu
	jp nz, options_bypass
	cp 1
	jr z, enable_anticrash
	cp 2
	jr z, disable_anticrash
options_shutdown:
	call destroyWindow
	jp power_down



inc_contrast:
	ld a, ($C008)						;current contrast value
	cp $1F
	ret z								;don't go above $1F
	inc a
	ld ($C008), a						;save new contrast level
	out (2), a							;write the new contrast level to the port
	ret
dec_contrast:
	ld a, ($C008)
	cp $00
	ret z								;don't go below $00
	dec a
	ld ($C008), a						;save new contrast level
	out (2), a							;write new contrast level to the port
	ret
enable_anticrash:
	ld hl, flagbyte
	set anticrash, (hl)					;anticrash = bit 7
	call save_flagbyte					;writeback the flag byte to the program variable
	jp options_bypass
disable_anticrash:
	ld hl, flagbyte
	res anticrash, (hl)
	call save_flagbyte
	jp options_bypass





create_launcher:						;creates launcher program if it doesn't exist
	ld hl, launcher_name-1
	push hl
	rst 20h
	rst 10h
	jr c, c_l_notpresent
	call _delvar						;delete it if it exists
c_l_notpresent:							;now there is no launcher prog
	pop hl
	rst 20h								;put name in OP1
	ld hl, launcher_code_end-launcher_code	;get length of launcher code in hl
	call _createprog					;make the launcher prog variable to ensure its existence, so user can't lose it
	ld a, b
	ex de, hl							;ld ahl, bde
	call decodeABS						;decode abs address of variable data
	add a, 64							;set bit 7, this is a RAM page
	out (5), a
	inc a
	out (6), a							;load RAM pages that variable resides in
	ld de, $4002
	add hl, de							;add offset into first page swap space
	ex de, hl							;put address into de, as a destination
	ld hl, launcher_code				;read from here
	ld bc, launcher_code_end-launcher_code	;get the length into bc
	ldir								;copy launcher code into the new launcher program
	ld a, $0d
	out (5), a
	call _clrLCD
	ld hl, $0503
	ld (_curRow), hl
	ld hl, installedtext
	call _puts							;'Installed'
	ld hl, $201C
	ld (_pencol), hl
	ld hl, runinstr1text
	call _vputs							;'To use File Manager'
	ld hl, $2618
	ld (_pencol), hl
	ld hl, runinstr2text
	call _vputs							;'run program 'FileMan''
	ret


clrLaunchExecName:						;clears the execname in launcher code
	ld hl, launcher_code_execname
	ld b, 9
clrLENloop:
	ld (hl), $6E
	inc hl
	djnz clrLENloop


quit:
	call _clrLCD
	res 4, (IY+$09)						;tell calc to ignore any on-key presses that occurred
	call _op1set0						;set OP1 to zero
	call _stoans						;store OP1 (zero) to Ans variable, to tell launcher to exit
	ret									;exit
	





	





;====================================================================
; DisplayHL:  (idea from SCaBBy/Mardell)    [Assembly Coder's Zenith]
;  Display HL as a 5-digit decimal number with no leading zeros
;  out: AF, HL, BC, DE destroyed
;====================================================================
DisplayHL:
 ld c,'0'  				; save ascii value for zero
 ld de,_OP1+5			; point to end of the buffer
 xor a				; zero terminate string
 ld (de),a				; set last byte to zero
DisplayHLl:
 call _divHLby10		; next digit
 dec de				; next position
 add a,c				; convert to ascii
 ld (de),a				; save digit
 ld a,h				; load upper byte
 or l					; check with lower byte for zero
 jr nz,DisplayHLl			; loop
 ex de,hl				; point to buffer
 call _vputs			; print number
 ret					; we're done



;String_compare
;by Cassady Roop
;INPUTS: HL - base string
;		 DE - comparing string
;		 B - number of bytes from base addresses to compare
;OUTPUTS: z flag set if strings match; nz if they don't
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
String_compare:
	ld a, (de)					;get value from one string into accumulator
	cp (hl)					;A-(HL)=A
	ret nz						;if they don't match, return with error
	inc hl
	inc de
	djnz String_compare		;reloop if not the last byte to compare
	ret							;return without error




;------------------------------------------------------------------------------
;TextIn custom-- Text input that handles upper, lowercase, and numeric chars.
;This version disallows space chars
;by Cassady Roop
;INPUT: hl- points to string storage area.
;		b-  max number of characters to accept and store, up to 255.
;OUTPUT: string stored to desired location.
;		 string displayed at pen location on the screen.
;		 if canceled with EXIT key, returns with nz. If successful, returns z.
;------------------------------------------------------------------------------
TextIn:
	push hl
	push bc
	call _getkey			;get keypress. it destroys hl.
	pop bc
	pop hl
	ld c, a
	cp kenter
	jp z, textin_commit		;enter was pressed
	cp kexit				;exit key?
	jr z, textin_cancel		;EXIT	
	ld a, b					;load counter value
	cp 0					;see if char allowance has maxed out
	jr z, TextIn			;too many characters, so bypass remaining input but not the enter and exit keys.
	ld a, c					;restore keycode to a
;	cp kSpace				;is it space?
;	jr z, textin_space		;it is a space
	push af					;preserve the keycode
	sub $1C					;k0
	jp m, TextIn_unknown	;it's not a known key if negative
	sub $0A					;k9 +1
	jp m, textin_number		;if neg, assumed to be a number key
	sub $02					;kCapA
	jp m, TextIn_unknown	;it's not a known key if negative
	sub $1A					;kCapZ +1
	jp m, textin_capletter	;if negative, then assume capital letter
	sub $1B					;kz +1
	jp m, textin_lowletter	;if neg, assume lower case letter
textin_unknown:				;go here if it is unknown
	pop af					;what gets pushed, must get popped
	jr TextIn				;must be an unknown key at this point
textin_space:
	ld a, Lspace			;char code for space
	ld (hl), a				;put into string storage
	call _vputmap			;display a space
	inc hl					;forward pointer
	dec b					;decrement counter
	jr TextIn				;re-loop
textin_number:
	pop af					;get the original keycode
	add a, 20				;keycode-->char code
	ld (hl), a				;store
	call _vputmap			;display the number
	inc hl					;inc storage pointer
	dec b					;dec counter
	jr TextIn
textin_capletter:
	pop af
	add a, 25				;keycode-->char code
	ld (hl), a				;store
	call _vputmap			;display the letter
	inc hl					;increment string pointer
	dec b					;dec counter
	jr TextIn
textin_lowletter:
	pop af
	add a, 31				;keycode-->char code
	ld (hl), a
	call _vputmap
	inc hl
	dec b
	jr TextIn
textin_commit:				;enter key pressed
	xor a					;a=0
	cp 0					;set the z flag
	ret						;done
textin_cancel:				;exit key pressed
	xor a					;a=0
	cp 1					;res the z flag
	ret						;return with error


;decodeABS   decodes absolute address in ahl.
;by Cassady Roop
;input - AHL = absolute address, like that outputted by _findsym
;output - A = page
;		 HL = offset into page.  Add $4000 for first swap window; $8000 for second swap window.
decodeABS:
	cp 0										;is a zero?
	ret z										;if so, hl is already a valid pointer
	dec a
	rlca										;*2
	rlca										;*4
	ld de, $4000								;length of one page
dABSloop:
	push hl										;save
	sbc hl, de									;hl=hl-de
	jr c, dABSdone								;if hl was less than $4000
	pop bc										;get rid of the value we saved
	inc a										;increment the page counter
	jr dABSloop									;reloop
dABSdone:
	inc a										;page correction
	pop hl										;the last pushed value is the offset
	ret											;we are done


;set_onChecksum
;calculates and loads correct value to _onChecksum
;taken from TI86 ROM 1.3 $0E0B
set_onChecksum:
	ld a, $42
	out (6), a
	push ix
	ld ix, $8000
	ld bc, $3FFF
	ld hl, $0008
	ld d, h
s_oC_loop:
	ld e, (ix+0)
	inc ix
	add hl, de
	dec bc
	ld a, c
	or b
	jr nz, s_oC_loop
	ld e, d
	ex de, hl
	sbc hl, de
	pop ix
	ld (_onChecksum), hl			;load new checksum
	ret



save_flagbyte:									;save the flagbyte to the original program so it can have lasting effects
	ld hl, self_name-1							;name of this program
	rst 20h										;move to OP1
	rst 10h										;call _findsym
	ld a, b
	ex de, hl									;ld ahl, bde
	call decodeABS								;get relative location
	add a, 64									;set bit to make it a RAM page
	out (5), a
	inc a
	out (6), a									;load pages to swap area
	ld de, $4000
	add hl, de									;offset into swap area
	inc hl
	inc hl										;skip length bytes
	inc hl
	inc hl										;skip 8E28 ASM token
	ld de, flagbyte-_asm_exec_ram				;offset from beginning of prog to the flagbyte
	add hl, de									;point to the flagbyte in original program
	ld a, (flagbyte)							;get current value
	ld (hl), a									;save current value
	ld a, $0D
	out (5), a									;put the call table back
	ret
	

makemenu:
 ld a, (hl)
 ld (tempbyte), a					;save menutable entries
 ld de, (_pencol)					;base coords
 push de
 ld a, (hl)
 inc hl
 ld b,a
menudisplayloop:
 call _vputs
 ld a,(_penRow)
 add a, 6
 ld (_penRow),a
 pop de
 push de
 ld a, e							;base column coord
 ld (_penCol),a
 djnz menudisplayloop
 pop de
 push de
 ld a, e							;get base column
 sub 4
 ld (_pencol),a
 ld a, d
 ld (_penrow), a
 ld a, 5
 call _vputmap
 ld a, (_pencol)
 sub 4
 ld (_pencol), a
 ld c, 1
menuloop:
 push bc
 call _get_key
 pop bc
 ld b, 3
 cp K_EXIT
 jp z,menuexit
 cp K_UP
 jr z,m_up
 cp K_DOWN
 jr z,m_down
 cp k_second
 jr z, m_enter
 cp K_ENTER
 jr nz,menuloop
m_enter:
 pop de
 ld a, c
 cp a
 ret
m_down:
 ld a,' '
 push bc
 call _vputmap
 pop bc
 djnz m_down
 ld a, (_penCol)
 sub 3
 ld (_pencol), a
 ld a,(tempbyte)
 rla
 ld b, a
 rla
 add a, b
 pop de
 push de
 add a, d								;add base row
 sub 6
 ld b,a
 ld a,(_penRow)
 cp b
 jr z,setmenu1
 inc c									;increment item counter
 add a, 6
 ld (_penRow),a
 jr menudisp
m_up:
 ld a,' '
 push bc
 call _vputmap
 pop bc
 djnz m_up
 ld a, (_pencol)
 sub 3
 ld (_pencol), a
 ld a,(_penRow)
 pop de
 push de
 cp d
 jr z,setmenumax
 dec c									;decrement item counter
 sub 6
 ld (_penRow),a
 jr menudisp
setmenu1:
 ld c, 1
 ld a, d
 ld (_penRow),a
 jr menudisp
setmenumax:
 ld a,(tempbyte)
 ld c, a
 rla
 ld b, a
 rla
 add a, b
 add a, d
 sub 6
 ld (_penRow),a
menudisp:
 ld a,5
 push bc
 call _vputmap
 pop bc
 ld a, (_pencol)
 sub 4
 ld (_pencol), a
 jp menuloop
menuexit:
 pop de
 ld a, 0
 cp 1
 ret





flagbyte:										;program state/options flags
	.db $00
titletext:
	.db "File Manager 1.2",0
credittext:
	.db "by Cassady Roop",0
showtext:
	.db "Show: ",0
prgmtext:
	.db "PRGM",0
strntext:
	.db "STRN",0
pagetext:
	.db "Page ",0
vartext:
	.db "Variable: ",0
copytext:
	.db "COPY",0
rentext:
	.db "RENAME",0
deltext:
	.db "DELETE",0
convtext:
	.db "CONVERT",0
infotext:
	.db "INFO",0
optionstext:
	.db "Options",0
adjusttext:
	.db "Adjust",0
f1text:
	.db "F1  -  ",0
totext:
	.db "to:",0
basictext:
	.db "BASIC ",0
asmtext:
	.db "ASM ",0
sizetext:
	.db "Size: ",0
bytestext:
	.db "bytes",0
typetext:
	.db "Type: ",0
runtext:
	.db "RUN ",0
errortext:
	.db "ERROR - ",0
notprgmtext:
	.db "NOT A PROGRAM",0
installedtext:
	.db "Installed",0
runinstr1text:
	.db "To use File Manager",0
runinstr2text:
	.db "run program 'FileMan'",0
locktext:
	.db "LOCK ",0
lockedtext:
	.db "ALREADY EDIT-LOCKED",0
notbasictext:
	.db "NOT A BASIC PRGM",0
overwritetext:
	.db "Overwrite?",0
confirmtext:
	.db "CONFIRM",0
toktext:
	.db "Tokenizing...",0
searchbase:								;base name to search from in alphabetic search of VAT for program vars
	.db $12,$08,"AAAAAAAA"				;type $12 (prgm), length $08, a bunch of a's.
temp_name:
	.db 8,"!",16,"FMan12"				;uncommon, hidden name
launcher_name:
	.db 7,"FileMan"						;name of BASIC launcher
self_name:								;name of this asm program
	.db 7,"fileman"


launcher_code:							;code for the tokenized BASIC launcher program
	.db $E0,$54,$6F,$70,$00,$6F,
	.db	$8E,$25,$10,$39,$66,$69,
	.db	$6C,$65,$6D,$61,$6E,$6F,
	.db	$D8,$3C,$03,$41,$6E,$73,
	.db	$50,$44,$30,$00,$6F,$DF,
	.db	$6F
launcher_code_execname:					;beginning of colon string to replace with name
	.db $6E,$6E,$6E,$6E,$6E,$6E,
	.db	$6E,$6E,$6E,$6F,$E1,$00,
	.db	$04,$54,$6F,$70,$00
launcher_code_end:


action_menutable:						;the actions menu
	.db 6								;6 items
	.db "RUN",0
	.db "COPY",0
	.db "RENAME",0
	.db "DELETE",0
	.db "CONVERT",0
	.db "INFO",0

convert_menutable:
	.db 2
	.db "PRGM",0
	.db "STRN",0

option_menutable:
	.db 3
	.db "Shut Down",0
	.db "Contrast",0
	.db "Anti-crash",0

anticrash_menutable:
	.db 2
ontext:
	.db "ON",0
offtext:
	.db "OFF",0




.end